# Generated by Django 4.2.16 on 2024-09-19 12:34

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('edx_name_affirmation', '0008_alter_historicalverifiedname_options'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='historicalverifiedname',
            options={'get_latest_by': 'history_date', 'ordering': ('-history_date', '-history_id'), 'verbose_name': 'historical verified name'},
        ),
        migrations.AddField(
            model_name='historicalverifiedname',
            name='platform_verification_attempt_id',
            field=models.PositiveIntegerField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='verifiedname',
            name='platform_verification_attempt_id',
            field=models.PositiveIntegerField(blank=True, null=True),
        ),
    ]
